//******************************************************************************************
//* Scherk-Collins Surface, by Paul Nylander, bugman123.com
//* runtime: 36 seconds
//******************************************************************************************

#declare nstory=10; #declare ntwist=1; #declare wflange=1.5; #declare rwarp=3.75; #declare rtube=rwarp/30; #declare theta0=pi*clock;

camera{location <0,0,10> look_at <0,0,0> right x*image_width/image_height up z sky <0,0,1>}
light_source{<0,0,10>,1}

//Basic Functions
#macro mod2(a,b) #local c=mod(a,b); (c>0?c:c+b) #end // well-behaved (always return positive)
#macro ln2(X) #if(X>0) #local Y=ln(X); #else #local Y=0; #end Y #end
#macro atan3(Y,X) #if(X=0 & Y=0) #local theta=0; #else #local theta=atan2(Y,X); #end theta #end
#declare colors=array[7] {<0,1,0,0>,<1/6,1,1,0>,<1/3,0,1,0>,<1/2,0,1,1>,<2/3,0,0,1>,<5/6,1,0,1>,<1,1,0,0>};
#macro hue(h) #local n=dimension_size(colors,1); #local i=0;
 #while(i<n & colors[min(i,n-1)].x<h) #local i=i+1; #end
 #if(i>0 & i<n) #local x1=colors[i-1].x; #local x2=colors[i].x; #local c=((x2-h)*colors[i-1]+(h-x1)*colors[i])/(x2-x1); #else #local c=colors[min(i,n-1)]; #end
 <c.y,c.z,c.t>
#end

//Complex Functions
#macro Complex(X,Y) <X,Y> #end
#declare I=Complex(0,1);
#macro Re(Z) Z.x #end
#macro Im(Z) Z.y #end
#macro Abs(Z) vlength(Z) #end
#macro Arg(Z) atan3(Im(Z),Re(Z)) #end
#macro Pow(Z,n) #local r=Abs(Z); #if(r=0) #local z2=Complex(0,0); #else #local theta=n*Arg(Z); #local z2=pow(r,n)*Complex(cos(theta),sin(theta)); #end z2 #end
#macro Sqrt(Z) Pow(Z,1/2) #end
#macro Mult(z1,z2) Complex(Re(z1)*Re(z2)-Im(z1)*Im(z2),Im(z1)*Re(z2)+Re(z1)*Im(z2)) #end
#macro Ln(Z) Complex(ln2(Abs(Z)),Arg(Z)) #end
#macro Cot(Z) Complex(-sin(2*Re(Z)),sinh(2*Im(Z)))/(cos(2*Re(Z))-cosh(2*Im(Z))) #end
#macro ArcTan(Z) #local z1=Complex(1,0)+Mult(I,Z); #local z2=Complex(1,0)-Mult(I,Z); Complex(Arg(z1)-Arg(z2),ln(Abs(z2))-ln(Abs(z1)))/2 #end

//Parametric Plotting, function f must be defined before this function can be called
#macro ParametricPlot3D(u1,u2,du, v1,v2,dv) #local imax=int((u2-u1)/du); #local jmax=int((v2-v1)/dv);
 #local Mesh=array[imax+1][jmax+1]; #local i=0;
 #while(i<=imax) #local U=u1+i*du; #local j=0; #while(j<=jmax) #local Mesh[i][j]=f(U,v1+j*dv); #local j=j+1; #end #local i=i+1; #end
 DrawMesh(Mesh)
#end
#macro pt(p) <p.x,p.y,p.z> #end
#macro det3(a,b,c,d,e,f,g,h,i) -c*e*g+b*f*g+c*d*h-a*f*h-b*d*i+a*e*i #end // 33 matrix
#macro color_triangle(p1,n1,c1, p2,n2,c2, p3,n3,c3) // adapted from Chris Colefax's triangle mapping macro
 #local nx=p2-p1; #local ny=p3-p1; #local nz=vcross(nx,ny);
 smooth_triangle{p1,n1,p2,n2,p3,n3 texture{
  #if(det3(nx.x,nx.y,nx.z, ny.x,ny.y,ny.z, nz.x,nz.y,nz.z)=0) pigment{rgb c1} #else pigment{
   average pigment_map{
    [1 gradient x color_map{[0 rgbt <0,0,0,0.5>][1 rgbt <3*c2.x,3*c2.y,3*c2.z,0.5>]}]
    [1 gradient y color_map{[0 rgbt <0,0,0,0.5>][1 rgbt <3*c3.x,3*c3.y,3*c3.z,0.5>]}]
    [1 gradient z color_map{[0 rgbt <0,0,0,0.5>][1 rgbt <3*c1.x,3*c1.y,3*c1.z,0.5>]}]
   }
   matrix <1.01,0,1, 0,1.01,1, 0,0,1, -0.002,-0.002,-1>
   matrix <nx.x,nx.y,nx.z, ny.x,ny.y,ny.z, nz.x,nz.y,nz.z, p1.x,p1.y,p1.z>
  } #end
  finish{reflection 0.4}
 }}
#end
#macro colorquad(p1,n1,c1, p2,n2,c2, p3,n3,c3, p4,n4,c4)
 #if(vlength(p3-p1)<vlength(p4-p2))
  color_triangle(p1,n1,c1, p2,n2,c2, p3,n3,c3) color_triangle(p1,n1,c1, p3,n3,c3, p4,n4,c4)
 #else
  color_triangle(p1,n1,c1, p2,n2,c2, p4,n4,c4) color_triangle(p2,n2,c2, p3,n3,c3, p4,n4,c4)
 #end
#end
#macro DrawMesh(Mesh)
 #local imax=dimension_size(Mesh,1); #local jmax=dimension_size(Mesh,2); #local normals=array[imax][jmax];
 #local i=0;
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=<0,0,0>; #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while(i<imax-1) #local j=0; // adapted from Tim Wenclawiak's Make_Normals function
  #while(j<jmax-1)
   #local p0=(Mesh[i][j]+Mesh[i][j+1]+Mesh[i+1][j]+Mesh[i+1][j+1])/4;
   #local N=vcross(pt(Mesh[i+1][j]-p0),pt(Mesh[i+1][j+1]-p0));
   #local E=vcross(pt(Mesh[i+1][j+1]-p0),pt(Mesh[i][j+1]-p0));
   #local S=vcross(pt(Mesh[i][j+1]-p0),pt(Mesh[i][j]-p0));
   #local W=vcross(pt(Mesh[i][j]-p0),pt(Mesh[i+1][j]-p0));
   #local normals[i][j]=normals[i][j]+S+W;
   #local normals[i+1][j]=normals[i+1][j]+N+W;
   #local normals[i][j+1]=normals[i][j+1]+S+E;
   #local normals[i+1][j+1]=normals[i+1][j+1]+N+E;
   #local j=j+1;
  #end
  #local i=i+1;
 #end
 #local i=0;
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=vnormalize(normals[i][j]); #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while (i<imax-1) #local j=0;
  #while (j<jmax-1)
   colorquad(
    pt(Mesh[i  ][j  ]),normals[i  ][j  ],hue(Mesh[i  ][j  ].t),
    pt(Mesh[i+1][j  ]),normals[i+1][j  ],hue(Mesh[i+1][j  ].t),
    pt(Mesh[i+1][j+1]),normals[i+1][j+1],hue(Mesh[i+1][j+1].t),
    pt(Mesh[i  ][j+1]),normals[i  ][j+1],hue(Mesh[i  ][j+1].t)
   )
   #local j=j+1;
  #end
  #local i=i+1;
 #end
#end                  

//Scherk-Collins Surface Calculations
#macro Twist(p,theta) <p.x*cos(theta)-p.y*sin(theta),p.x*sin(theta)+p.y*cos(theta),p.z> #end
#macro Warp(p,theta) <(p.x+rwarp)*cos(theta),(p.x+rwarp)*sin(theta),p.y> #end
#macro Scherk(Z) #local t1=Sqrt(2*Cot(Z)); #local t2=Cot(Z)+Complex(1,0);
 <xsign*Re(0.5*(Ln(t1-t2)-Ln(t1+t2)-I*pi)/sqrt(2)), ysign*Re(Mult(I,ArcTan(Complex(1,0)-t1)-ArcTan(Complex(1,0)+t1)+Complex(pi,0))/sqrt(2)), Re(Z)>
#end
#macro f(X,Y) #local theta=4*X/nstory; #local p=Warp(Twist(Scherk(Complex(X,Y)),theta0+ntwist*theta),theta); <p.x,p.y,p.z,(X-x1)/(x2-x1)> #end
#declare x1=0; #declare x2=nstory*pi/2; #declare dx=pi/8;
#declare y1=1e-10; #declare y2=wflange/2; #declare dy=(y2-y1)/6;
#declare xsign= 1; #declare ysign= 1; ParametricPlot3D(x1,x2,dx, y1,y2,dy)
#declare xsign= 1; #declare ysign=-1; ParametricPlot3D(x1,x2,dx, y1,y2,dy)
#declare xsign=-1; #declare ysign= 1; ParametricPlot3D(x1,x2,dx, y1,y2,dy)
#declare xsign=-1; #declare ysign=-1; ParametricPlot3D(x1,x2,dx, y1,y2,dy)
